"use strict";

exports.__esModule = true;
exports.ApolloProvider = ApolloProvider;
exports.useApolloClient = useApolloClient;
exports.useApolloQuery = useApolloQuery;
exports.useApolloMutation = useApolloMutation;

var _react = _interopRequireWildcard(require("react"));

var _reactFastCompare = _interopRequireDefault(require("react-fast-compare"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

var ApolloContext = _react.default.createContext();

function ApolloProvider(_ref) {
  var children = _ref.children,
      client = _ref.client;
  return _react.default.createElement(ApolloContext.Provider, {
    value: client
  }, children);
}

function useApolloClient() {
  return (0, _react.useContext)(ApolloContext);
}

function useApolloQuery(query, _temp) {
  var _ref2 = _temp === void 0 ? {} : _temp,
      variables = _ref2.variables,
      apolloContextOptions = _ref2.context,
      restOptions = _objectWithoutPropertiesLoose(_ref2, ["variables", "context"]);

  var client = useApolloClient();

  var _useState = (0, _react.useState)(),
      result = _useState[0],
      setResult = _useState[1];

  var previousQuery = (0, _react.useRef)(); // treat variables and context options separately because they are objects
  // and the other options are JS primitives

  var previousVariables = (0, _react.useRef)();
  var previousApolloContextOptions = (0, _react.useRef)();
  var previousRestOptions = (0, _react.useRef)();
  var observableQuery = (0, _react.useRef)();
  (0, _react.useEffect)(function () {
    var subscription = observableQuery.current.subscribe(function (nextResult) {
      setResult(nextResult);
    });
    return function () {
      subscription.unsubscribe();
    };
  }, [query, objToKey(variables), objToKey(previousApolloContextOptions), objToKey(restOptions)]);
  var helpers = {
    fetchMore: function fetchMore(opts) {
      return observableQuery.current.fetchMore(opts);
    }
  };

  if (!(query === previousQuery.current && (0, _reactFastCompare.default)(variables, previousVariables.current) && (0, _reactFastCompare.default)(apolloContextOptions, previousApolloContextOptions.current) && (0, _reactFastCompare.default)(restOptions, previousRestOptions.current))) {
    previousQuery.current = query;
    previousVariables.current = variables;
    previousApolloContextOptions.current = apolloContextOptions;
    previousRestOptions.current = restOptions;
    var watchedQuery = client.watchQuery(_extends({
      query: query,
      variables: variables
    }, restOptions));
    observableQuery.current = watchedQuery;
    var currentResult = watchedQuery.currentResult();

    if (currentResult.partial) {
      // throw a promise - use the react suspense to wait until the data is
      // available
      throw watchedQuery.result();
    }

    setResult(currentResult);
    return _extends({}, helpers, currentResult);
  }

  return _extends({}, helpers, result);
}

function useApolloMutation(mutation, baseOptions) {
  var client = useApolloClient();
  return function (localOptions) {
    return client.mutate(_extends({
      mutation: mutation
    }, baseOptions, localOptions));
  };
}

function objToKey(obj) {
  if (!obj) {
    return null;
  }

  var keys = Object.keys(obj);
  keys.sort();
  var sortedObj = keys.reduce(function (result, key) {
    result[key] = obj[key];
    return result;
  }, {});
  return JSON.stringify(sortedObj);
}